@@ -38,7 +38,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.9.2.16 2002/09/10 22:33:27 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.20 2002/02/16 18:23:49 shadow Exp $");
 
 #include "../afs/sysincludes.h" /*Standard vendor system headers*/
 #include "../afs/afsincludes.h" /*AFS-based standard headers*/
@@ -132,7 +132,7 @@ int afs_FlushVCache(struct vcache *avc, int *slept)
 	       ICL_TYPE_INT32, avc->states);
 #ifdef  AFS_OSF_ENV
     AFS_GUNLOCK();
-    VN_LOCK(AFSTOV(avc));
+    VN_LOCK((struct vnode *)avc);
     AFS_GLOCK();
 #endif
 
@@ -214,15 +214,15 @@ int afs_FlushVCache(struct vcache *avc, int *slept)
     afs_vcount--;
     vSetType(avc, VREG);
     if (VREFCOUNT(avc) > 0) {
-        VN_UNLOCK(AFSTOV(avc));
-        AFS_RELE(AFSTOV(avc));
+        VN_UNLOCK((struct vnode *)avc);
+        AFS_RELE((struct vnode *)avc);
     } else {
        if (afs_norefpanic) {
 	  printf ("flush vc refcnt < 1");
 	  afs_norefpanic++;
 	  (void) vgone(avc, VX_NOSLEEP, (struct vnodeops *) 0);
 	  AFS_GLOCK();
-	  VN_UNLOCK(AFSTOV(avc));
+	  VN_UNLOCK((struct vnode *)avc);
        }
        else osi_Panic ("flush vc refcnt < 1");
     }
@@ -232,7 +232,7 @@ int afs_FlushVCache(struct vcache *avc, int *slept)
 
 bad:
 #ifdef	AFS_OSF_ENV
-    VN_UNLOCK(AFSTOV(avc));
+    VN_UNLOCK((struct vnode *)avc);
 #endif
     return code;
 
@@ -251,7 +251,7 @@ void afs_InactiveVCache(struct vcache *avc, struct AFS_UCRED *acred)
     AFS_STATCNT(afs_inactive);
     if (avc->states & CDirty) {
       /* we can't keep trying to push back dirty data forever.  Give up. */
-      afs_InvalidateAllSegments(avc, 1/*set lock*/);  /* turns off dirty bit */
+      afs_InvalidateAllSegments(avc);  /* turns off dirty bit */
     }
     avc->states	&= ~CMAPPED;	/* mainly used by SunOS 4.0.x */
     avc->states	&= ~CDirty;	/* Turn it off */
@@ -466,85 +466,6 @@ static afs_int32 afs_QueueVCB(struct vcache *avc)
     return 0;
 }
 
-#ifdef AFS_LINUX22_ENV
-/* afs_TryFlushDcacheChildren -- Shakes loose vcache references held by
- *                               children of the dentry
- *
- * LOCKS -- Called with afs_xvcache write locked. Drops and reaquires
- *          AFS_GLOCK, so it can call dput, which may call iput, but
- *          keeps afs_xvcache exclusively.
- *
- * Tree traversal algorithm from fs/dcache.c: select_parent()
- */
-static void afs_TryFlushDcacheChildren(struct vcache *tvc)
-{
-    struct inode *ip = AFSTOI(tvc);
-    struct dentry *this_parent;
-    struct list_head *next;
-    struct list_head *cur;
-    struct list_head *head = &ip->i_dentry;
-    struct dentry *dentry;
-    
-restart:
-    DLOCK();
-    cur = head;
-    while ((cur = cur->next) != head) {
-	dentry = list_entry(cur, struct dentry, d_alias);
-#ifdef notdef
-	if (DCOUNT(dentry)) {
-	    this_parent = dentry;
-	repeat:
-	    next = this_parent->d_subdirs.next;
-	resume:
-	    while (next && next != &this_parent->d_subdirs) {
-		struct list_head *tmp = next;
-		struct dentry *dchld = list_entry(tmp, struct dentry, d_child);
-		
-		next = tmp->next;
-		if (!DCOUNT(dchld) && !dchld->d_inode) {
-		    DGET(dchld);
-		    AFS_GUNLOCK();
-		    DUNLOCK();
-		    d_drop(dchld);
-		    dput(dchld);
-		    AFS_GLOCK();
-		    DLOCK();
-		    goto repeat;
-		}
-		/*
-		 * Descend a level if the d_subdirs list is non-empty.
-		 */
-		if (!list_empty(&dchld->d_subdirs)) {
-		    this_parent = dchld;
-		    goto repeat;
-		}
-	    }
-	    
-	    /*
-	     * All done at this level ... ascend and resume the search.
-	     */
-	    if (this_parent != dentry) {
-		next = this_parent->d_child.next;
-		this_parent = this_parent->d_parent;
-		goto resume;
-	    }
-	}
-#endif
-
-	if (!DCOUNT(dentry)) {
-	    AFS_GUNLOCK();
-	    DGET(dentry);
-	    DUNLOCK();
-	    d_drop(dentry);
-	    dput(dentry);
-	    AFS_GLOCK();
-	    goto restart;
-	}
-    }
-    DUNLOCK();
-
-}
-#endif /* AFS_LINUX22_ENV */
 
 /*
  * afs_RemoveVCB
@@ -638,6 +559,63 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
     int code, fv_slept;
 
     AFS_STATCNT(afs_NewVCache);
+#ifdef AFS_LINUX22_ENV
+    if (!freeVCList) {
+	/* Free some if possible. */
+        struct afs_q *tq, *uq;
+        int i; char *panicstr;
+        int vmax = 2 * afs_cacheStats;
+        int vn = VCACHE_FREE;
+	
+        i = 0;
+        for(tq = VLRU.prev; tq != &VLRU && vn > 0; tq = uq) {
+	    tvc = QTOV(tq);
+	    uq = QPrev(tq);
+	    if (tvc->states & CVFlushed) 
+                refpanic ("CVFlushed on VLRU");
+	    else if (i++ > vmax)
+                refpanic ("Exceeded pool of AFS vnodes(VLRU cycle?)");
+	    else if (QNext(uq) != tq)
+                refpanic ("VLRU inconsistent");
+	    
+	    if (tvc == afs_globalVp)
+		continue;
+	    
+	    if ( VREFCOUNT(tvc) && tvc->opens == 0 ) {
+		struct inode *ip = (struct inode*)tvc;
+		if (list_empty(&ip->i_dentry)) {
+		    vn --;
+		}
+		else {
+		    struct list_head *cur;
+		    struct list_head *head = &ip->i_dentry;
+		    int all = 1;
+		restart:
+		    DLOCK();
+		    cur = head;
+		    while ((cur = cur->next) != head) {
+			struct dentry *dentry = list_entry(cur, struct dentry, d_alias);
+			if (!DCOUNT(dentry)) {
+			    AFS_GUNLOCK();
+			    DGET(dentry);
+			    DUNLOCK();
+			    d_drop(dentry);
+			    dput(dentry);
+			    AFS_GLOCK();
+			    goto restart;
+			}
+			else {
+			    all = 0;
+			}
+		    }
+		    DUNLOCK();
+		    if (all) vn --;
+		}
+	    }
+	    if (tq == uq) break;
+        }
+    }
+#endif /* AFS_LINUX22_ENV */
 #ifdef	AFS_OSF_ENV
 #ifdef	AFS_OSF30_ENV
     if (afs_vcount >= afs_maxvcount) 
@@ -729,11 +707,6 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
                }
             }
 #endif
-#if defined(AFS_LINUX22_ENV)
-	    if (tvc != afs_globalVp && VREFCOUNT(tvc) && tvc->opens == 0)
-		afs_TryFlushDcacheChildren(tvc);
-#endif
-
 	   if (VREFCOUNT(tvc) == 0 && tvc->opens == 0
 	       && (tvc->states & CUnlinkedDel) == 0) {
 		code = afs_FlushVCache(tvc, &fv_slept);
@@ -827,7 +800,7 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
     hzero(tvc->m.DataVersion);		/* in case we copy it into flushDV */
 #ifdef	AFS_OSF_ENV
     /* Hold it for the LRU (should make count 2) */
-    VN_HOLD(AFSTOV(tvc));
+    VN_HOLD((struct vnode *)tvc);
 #else	/* AFS_OSF_ENV */
     VREFCOUNT_SET(tvc, 1);	/* us */
 #endif	/* AFS_OSF_ENV */
@@ -896,12 +869,12 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
     AFS_VN_INIT_BUF_LOCK(&(tvc->v));
 #endif
 #else
-    SetAfsVnode(AFSTOV(tvc));
+    SetAfsVnode((struct vnode *)tvc);
 #endif /* AFS_SGI64_ENV */
 #ifdef AFS_DARWIN_ENV
     tvc->v.v_ubcinfo = UBC_INFO_NULL;
     lockinit(&tvc->rwlock, PINOD, "vcache rwlock", 0, 0);
-    cache_purge(AFSTOV(tvc)); 
+    cache_purge((struct vnode *)tvc); 
     tvc->v.v_data=tvc;
     tvc->v.v_tag=VT_AFS;
     /* VLISTNONE(&tvc->v); */
@@ -911,11 +884,16 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 #endif 
 #ifdef AFS_FBSD_ENV
     lockinit(&tvc->rwlock, PINOD, "vcache rwlock", 0, 0);
-    cache_purge(AFSTOV(tvc)); 
+    cache_purge((struct vnode *)tvc); 
     tvc->v.v_data=tvc;
     tvc->v.v_tag=VT_AFS;
+#ifndef VSTANDARD
     tvc->v.v_usecount++; /* steal an extra ref for now so vfree never happens */
-                         /* This extra ref is dealt with above... */
+                         /* this will prevent the vfs layer from reusing
+                          * afs vnodes, but afs will eventually run out of
+                          * vcache's and panic...
+                          */
+#endif
 #endif
     /*
      * The proper value for mvstat (for root fids) is setup by the caller.
@@ -967,7 +945,7 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 #endif /* AFS_SGI_ENV */
 #if defined(AFS_LINUX22_ENV)
     {
-	struct inode *ip = AFSTOI(tvc);
+	struct inode *ip = (struct inode*)tvc;
 	sema_init(&ip->i_sem, 1);
 #if defined(AFS_LINUX24_ENV)
 	sema_init(&ip->i_zombie, 1);
@@ -987,9 +965,6 @@ struct vcache *afs_NewVCache(struct VenusFid *afid, struct server *serverp,
 	INIT_LIST_HEAD(&ip->i_devices);
 #endif
 	ip->i_data.host = (void*) ip;
-#ifdef STRUCT_ADDRESS_SPACE_HAS_GFP_MASK
-	ip->i_data.gfp_mask = GFP_HIGHUSER;
-#endif
 	ip->i_mapping = &ip->i_data;
 #ifdef STRUCT_INODE_HAS_I_TRUNCATE_SEM
 	init_rwsem(&ip->i_truncate_sem);
@@ -1172,7 +1147,7 @@ afs_FlushActiveVcaches(doflocks)
 #ifdef	AFS_GFS_ENV
 		    VREFCOUNT_DEC(tvc);
 #else
-		    AFS_RELE(AFSTOV(tvc));
+		    AFS_RELE((struct vnode *)tvc);
 #endif
 		    /* Matches write code setting CCore flag */
 		    crfree(cred);
@@ -1278,6 +1253,7 @@ afs_SimpleVStat(avc, astat, areq)
     struct vrequest *areq;
 { /*afs_SimpleVStat*/
 
+    afs_size_t length;
     AFS_STATCNT(afs_SimpleVStat);
 
 #ifdef AFS_SGI_ENV
@@ -1288,16 +1264,21 @@ afs_SimpleVStat(avc, astat, areq)
 #endif
 
 	{
+#ifdef AFS_64BIT_ClIENT
+	    FillInt64(length, astat->Length_hi, astat->Length);
+#else /* AFS_64BIT_CLIENT */
+	    length = astat->Length;
+#endif /* AFS_64BIT_CLIENT */
 #if defined(AFS_SGI_ENV)
 	    osi_Assert((valusema(&avc->vc_rwlock) <= 0) &&
 		   (OSI_GET_LOCKID() == avc->vc_rwlockid));
-	    if (astat->Length < avc->m.Length) {
+	    if (length < avc->m.Length) {
 		vnode_t *vp = (vnode_t *)avc;
 		
 		osi_Assert(WriteLocked(&avc->lock));
 		ReleaseWriteLock(&avc->lock);
 		AFS_GUNLOCK();
-		PTOSSVP(vp, (off_t)astat->Length, (off_t)MAXLONG);
+		PTOSSVP(vp, (off_t)length, (off_t)MAXLONG);
 		AFS_GLOCK();
 		ObtainWriteLock(&avc->lock,67);
 	    }
@@ -1305,9 +1286,9 @@ afs_SimpleVStat(avc, astat, areq)
 	    /* if writing the file, don't fetch over this value */
 	    afs_Trace3(afs_iclSetp, CM_TRACE_SIMPLEVSTAT,
 		       ICL_TYPE_POINTER, avc,
-		       ICL_TYPE_INT32, avc->m.Length,
-		       ICL_TYPE_INT32, astat->Length);
-	    avc->m.Length = astat->Length;
+		       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length),
+		       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(length));
+	    avc->m.Length = length;
 	    avc->m.Date = astat->ClientModTime;
 	}
     avc->m.Owner = astat->Owner;
@@ -1386,7 +1367,7 @@ afs_WriteVCache(avc, astatus, areq)
 
     AFS_STATCNT(afs_WriteVCache);
     afs_Trace2(afs_iclSetp, CM_TRACE_WVCACHE, ICL_TYPE_POINTER, avc,
-	       ICL_TYPE_INT32, avc->m.Length);
+	       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length));
 
     do {
 	tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
@@ -1461,8 +1442,14 @@ afs_ProcessFS(avc, astat, areq)
 { /*afs_ProcessFS*/
 
     register int i;
+    afs_size_t length;
     AFS_STATCNT(afs_ProcessFS);
 
+#ifdef AFS_64BIT_CLIENT
+    FillInt64(length, astat->Length_hi, astat->Length);
+#else /* AFS_64BIT_CLIENT */
+    length = astat->Length;
+#endif /* AFS_64BIT_CLIENT */
     /* WARNING: afs_DoBulkStat uses the Length field to store a sequence
      * number for each bulk status request. Under no circumstances
      * should afs_DoBulkStat store a sequence number if the new
@@ -1480,9 +1467,9 @@ afs_ProcessFS(avc, astat, areq)
 	     *  values.
 	     */
 	    afs_Trace3(afs_iclSetp, CM_TRACE_PROCESSFS, ICL_TYPE_POINTER, avc,
-		       ICL_TYPE_INT32, avc->m.Length,
-		       ICL_TYPE_INT32, astat->Length);
-	    avc->m.Length = astat->Length;
+		       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length),
+		       ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(length));
+	    avc->m.Length = length;
 	    avc->m.Date = astat->ClientModTime;
 	}
     hset64(avc->m.DataVersion, astat->dataVersionHigh, astat->DataVersion);
@@ -1499,16 +1486,9 @@ afs_ProcessFS(avc, astat, areq)
 	avc->m.Mode |= S_IFDIR;
     }
     else if (astat->FileType == SymbolicLink) {
-	if (afs_fakestat_enable && (avc->m.Mode & 0111) == 0) {
-	    vSetType(avc, VDIR);
-	    avc->m.Mode |= S_IFDIR;
-	} else {
-	    vSetType(avc, VLNK);
-	    avc->m.Mode |= S_IFLNK;
-	}
-	if ((avc->m.Mode & 0111) == 0) {
-	    avc->mvstat = 1;
-	}
+	vSetType(avc, VLNK);
+	avc->m.Mode |= S_IFLNK;
+	if ((avc->m.Mode & 0111) == 0) avc->mvstat = 1;
     }
     avc->anyAccess = astat->AnonymousAccess;
 #ifdef badidea
@@ -1539,9 +1519,6 @@ afs_ProcessFS(avc, astat, areq)
 #ifdef AFS_LINUX22_ENV
     vcache2inode(avc);    /* Set the inode attr cache */
 #endif
-#ifdef AFS_DARWIN_ENV
-   osi_VM_Setup(avc,1);
-#endif
 
 } /*afs_ProcessFS*/
 
@@ -1683,7 +1660,7 @@ loop:
 #endif
 	ReleaseWriteLock(&tvc->lock);
 #ifdef AFS_DARWIN_ENV
-        osi_VM_Setup(tvc,0);
+        osi_VM_Setup(tvc);
 #endif
 	return tvc;
     }
@@ -1748,6 +1725,9 @@ loop:
     }
 
     ReleaseWriteLock(&tvc->lock);
+#ifdef AFS_DARWIN_ENV
+    osi_VM_Setup(avc);
+#endif
     return tvc;
 
 } /*afs_GetVCache*/
@@ -1903,6 +1883,9 @@ struct vcache *afs_LookupVCache(struct VenusFid *afid, struct vrequest *areq,
     afs_ProcessFS(tvc, &OutStatus, areq);
 
     ReleaseWriteLock(&tvc->lock);
+#ifdef AFS_DARWIN_ENV
+    osi_VM_Setup(tvc);
+#endif
     return tvc;
 
 }
@@ -1959,7 +1942,7 @@ struct vcache *afs_GetRootVCache(struct VenusFid *afid,
 	     * can be safely implemented */
 	    int vg;
 	    AFS_GUNLOCK();
-            vg = vget(AFSTOV(tvc));   /* this bumps ref count */
+            vg = vget((struct vnode *)tvc);   /* this bumps ref count */
 	    AFS_GLOCK();
 	    if (vg)
                 continue;
@@ -2410,7 +2393,7 @@ struct vcache *afs_FindVCache(struct VenusFid *afid, afs_int32 lockit,
             /* Grab this vnode, possibly reactivating from the free list */
 	    int vg;
 	    AFS_GUNLOCK();
-            vg = vget(AFSTOV(tvc));
+            vg = vget((struct vnode *)tvc);
 	    AFS_GLOCK();
 	    if (vg)
                 continue;
@@ -2472,7 +2455,7 @@ struct vcache *afs_FindVCache(struct VenusFid *afid, afs_int32 lockit,
 #endif
 #ifdef AFS_DARWIN_ENV
     if (tvc)
-        osi_VM_Setup(tvc, 0);
+        osi_VM_Setup(tvc);
 #endif
     return tvc;
 } /*afs_FindVCache*/
@@ -2532,7 +2515,7 @@ afs_int32 afs_NFSFindVCache(avcp, afid, lockit)
             /* Grab this vnode, possibly reactivating from the free list */
 	    int vg;
 	    AFS_GUNLOCK();
-            vg = vget(AFSTOV(tvc));
+            vg = vget((struct vnode *)tvc);
 	    AFS_GLOCK();
 	    if (vg) {
 		/* This vnode no longer exists. */
@@ -2544,8 +2527,8 @@ afs_int32 afs_NFSFindVCache(avcp, afid, lockit)
 		/* Duplicates */
 #ifdef AFS_OSF_ENV
 		/* Drop our reference counts. */
-		vrele(AFSTOV(tvc));
-		vrele(AFSTOV(found_tvc));
+		vrele((struct vnode *)tvc);
+		vrele((struct vnode *)found_tvc);
 #endif
 		afs_duplicate_nfs_fids++;
 		ReleaseSharedLock(&afs_xvcache);
@@ -2714,7 +2697,7 @@ void shutdown_vcache(void)
 		tvc->mvid = (struct VenusFid*)0;
 	    }
 #ifdef	AFS_AIX_ENV
-	    aix_gnode_rele(AFSTOV(tvc));
+	    aix_gnode_rele((struct vnode *)tvc);
 #endif
 	    if (tvc->linkData) {
 		afs_osi_Free(tvc->linkData, strlen(tvc->linkData)+1);
