@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/bozo/bosoprocs.c,v 1.5 2001/07/12 19:58:26 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/bozo/bosoprocs.c,v 1.11 2002/02/21 04:29:22 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -60,11 +60,11 @@ struct ktime *aktime; {
     code = 0;		/* assume success */
     switch (atype) {
       case 1:
-	bcopy(&bozo_nextRestartKT, aktime, sizeof(struct ktime));
+	memcpy(aktime, &bozo_nextRestartKT, sizeof(struct ktime));
 	break;
 
       case 2:
-	bcopy(&bozo_nextDayKT, aktime, sizeof(struct ktime));
+	memcpy(aktime, &bozo_nextDayKT, sizeof(struct ktime));
 	break;
 
       default:
@@ -92,11 +92,11 @@ struct ktime *aktime; {
     code = 0;		/* assume success */
     switch (atype) {
       case 1:
-	bcopy(aktime, &bozo_nextRestartKT, sizeof(struct ktime));
+	memcpy(&bozo_nextRestartKT, aktime, sizeof(struct ktime));
 	break;
 
       case 2:
-	bcopy(aktime, &bozo_nextDayKT, sizeof(struct ktime));
+	memcpy(&bozo_nextDayKT, aktime, sizeof(struct ktime));
 	break;
 
       default:
@@ -147,7 +147,6 @@ BOZO_GetDates(acall, aname, atime, abakTime, aoldTime)
 struct rx_call *acall;
 char *aname;
 afs_int32 *atime, *abakTime, *aoldTime; {
-    register afs_int32 code;
     struct stat tstat;
     char *strp;
     char tbuffer[AFSDIR_PATH_MAX];
@@ -363,6 +362,7 @@ char *aname; {
     struct afsconf_cell tcell;
     register afs_int32 code;
     char caller[MAXKTCNAMELEN];
+    char clones[MAXHOSTSPERCELL];
     
     if (!afsconf_SuperUser(bozo_confdir, acall, caller)) {
       code = BZACCESS;
@@ -370,7 +370,7 @@ char *aname; {
     }
     if (DoLogging) bozo_Log("%s is executing SetCellName '%s'\n", caller, aname);
 
-    code = afsconf_GetCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell);
+    code = afsconf_GetExtendedCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell, &clones);
     if (code) 
       goto fail;
 
@@ -383,7 +383,7 @@ char *aname; {
     }
 
     strcpy(tcell.name, aname);
-    code = afsconf_SetCellInfo(bozo_confdir, AFSDIR_SERVER_ETC_DIRPATH, &tcell);
+    code = afsconf_SetExtendedCellInfo(bozo_confdir, AFSDIR_SERVER_ETC_DIRPATH, &tcell, &clones);
 
   fail:
     osi_auditU (acall, BOS_SetCellEvent, code, AUD_STR, aname, AUD_END);
@@ -412,13 +412,14 @@ char **aname; {
 
 BOZO_GetCellHost(acall, awhich, aname)
 struct rx_call *acall;
-afs_int32 awhich;
+afs_uint32 awhich;
 char **aname; {
     register afs_int32 code;
     struct afsconf_cell tcell;
     register char *tp;
+    char clones[MAXHOSTSPERCELL];
 
-    code = afsconf_GetCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell);
+    code = afsconf_GetExtendedCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell, &clones);
     if (code) goto fail;
 
     if (awhich >= tcell.numServers) {
@@ -427,8 +428,13 @@ char **aname; {
     }
     
     tp = tcell.hostName[awhich];
-    *aname = (char *) malloc(strlen(tp)+1);
-    strcpy(*aname, tp);
+    *aname = (char *) malloc(strlen(tp)+3);
+    if (clones[awhich]) {
+	strcpy(*aname, "[");
+	strcat(*aname, tp);
+	strcat(*aname, "]");
+    } else
+        strcpy(*aname, tp);
     goto done;
     
 fail:
@@ -447,6 +453,7 @@ char *aname; {
     afs_int32 which;
     register int i;
     char caller[MAXKTCNAMELEN];
+    char clones[MAXHOSTSPERCELL];
 
     if (!afsconf_SuperUser(bozo_confdir, acall, caller)) {
       code = BZACCESS;
@@ -455,7 +462,7 @@ char *aname; {
     if (DoLogging) 
       bozo_Log("%s is executing DeleteCellHost '%s'\n", caller, aname);
 
-    code = afsconf_GetCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell);
+    code = afsconf_GetExtendedCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell, &clones);
     if (code) 
       goto fail;
 
@@ -472,9 +479,9 @@ char *aname; {
       goto fail;
     }
 
-    bzero(&tcell.hostAddr[which], sizeof(struct sockaddr_in));
-    bzero(tcell.hostName[which], MAXHOSTCHARS);
-    code = afsconf_SetCellInfo(bozo_confdir, AFSDIR_SERVER_ETC_DIRPATH, &tcell);
+    memset(&tcell.hostAddr[which], 0, sizeof(struct sockaddr_in));
+    memset(tcell.hostName[which], 0, MAXHOSTCHARS);
+    code = afsconf_SetExtendedCellInfo(bozo_confdir, AFSDIR_SERVER_ETC_DIRPATH, &tcell, &clones);
 
   fail:
     osi_auditU( acall, BOS_DeleteHostEvent, code, AUD_STR, aname, AUD_END);
@@ -489,6 +496,9 @@ char *aname; {
     afs_int32 which;
     register int i;
     char caller[MAXKTCNAMELEN];
+    char clones[MAXHOSTSPERCELL];
+    char *n;
+    char isClone = 0;
 
     if (!afsconf_SuperUser(bozo_confdir, acall, caller)) {
       code = BZACCESS;
@@ -497,13 +507,20 @@ char *aname; {
     if (DoLogging) 
       bozo_Log("%s is executing AddCellHost '%s'\n", caller, aname);
 
-    code = afsconf_GetCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell);
+    code = afsconf_GetExtendedCellInfo(bozo_confdir, (char *) 0, (char *) 0, &tcell, &clones);
     if (code) 
       goto fail;
 
+    n = aname;
+    if (*n == '[') {
+        *(n + strlen(n) -1) = 0;
+        ++n;
+        isClone = 1;
+    }
+
     which = -1;
     for(i=0;i<tcell.numServers;i++) {
-	if (strcmp(tcell.hostName[i], aname) == 0) {
+	if (strcmp(tcell.hostName[i], n) == 0) {
 	    which = i;
 	    break;
 	}
@@ -534,9 +551,10 @@ char *aname; {
 	}
     }
 
-    bzero(&tcell.hostAddr[which], sizeof(struct sockaddr_in));
-    strcpy(tcell.hostName[which], aname);
-    code = afsconf_SetCellInfo(bozo_confdir, AFSDIR_SERVER_ETC_DIRPATH, &tcell);
+    memset(&tcell.hostAddr[which], 0, sizeof(struct sockaddr_in));
+    strcpy(tcell.hostName[which], n);
+    clones[which] = isClone;
+    code = afsconf_SetExtendedCellInfo(bozo_confdir, AFSDIR_SERVER_ETC_DIRPATH, &tcell, &clones);
 
   fail:
     osi_auditU(acall, BOS_AddHostEvent, code, AUD_STR, aname, AUD_END);
@@ -572,7 +590,7 @@ struct bozo_key *akey;
       goto fail;
     }
     *akvno = tkeys.key[an].kvno;
-    bzero(akeyinfo, sizeof(struct bozo_keyInfo));
+    memset(akeyinfo, 0, sizeof(struct bozo_keyInfo));
 
     noauth = afsconf_GetNoAuthFlag(bozo_confdir);
     rxkad_GetServerInfo(acall->conn, &enc_level, 0, 0, 0, 0, 0);
@@ -581,9 +599,9 @@ struct bozo_key *akey;
      */
 
     if ((noauth) || (enc_level == rxkad_crypt)) {
-	bcopy(tkeys.key[an].key, akey, 8);
+	memcpy(akey, tkeys.key[an].key, 8);
     }
-    else bzero (akey, 8);
+    else memset(akey, 0, 8);
 
     code = stat(AFSDIR_SERVER_KEY_FILEPATH, &tstat);
     if (code == 0) {
@@ -1191,6 +1209,7 @@ int initBosEntryStats()
   bozo_bosEntryStats[7].path = AFSDIR_SERVER_KEY_FILEPATH;
   bozo_bosEntryStats[8].path = AFSDIR_SERVER_ULIST_FILEPATH;
 
+  return 0;
 }
 /* StatEachEntry - If it's not there, it is okay.  If anything else goes wrong
  * complain.  Otherwise check permissions: shouldn't allow write or (usually)
@@ -1281,7 +1300,7 @@ BOZO_GetInstanceInfo(acall, ainstance, atype, astatus)
 	strcpy(*atype, tb->type->name);
     else
 	(*atype)[0] = 0;    /* null string */
-    bzero(astatus, sizeof(struct bozo_status));	/* good defaults */
+    memset(astatus, 0, sizeof(struct bozo_status));	/* good defaults */
     astatus->goal = tb->goal;
     astatus->fileGoal = tb->fileGoal;
     astatus->procStartTime = tb->procStartTime;
@@ -1316,7 +1335,7 @@ char **aparm; {
     bnode_Hold(tb);
     if (anum == 999) {
 	if (tb->notifier) {
-	    bcopy(tb->notifier, tp, strlen(tb->notifier)+1);
+	    memcpy(tp, tb->notifier, strlen(tb->notifier)+1);
 	    code = 0;
 	} else
 	    code = BZNOENT;	/* XXXXX */
