@@ -13,7 +13,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.6 2001/07/12 19:59:27 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.8 2001/11/01 04:02:31 shadow Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -310,6 +310,7 @@ static DisplayRecord(outFilep, alp, rsize)
 	    printfParms[pfpix++] = alp[pix];
 	    break;
 	case ICL_TYPE_HYPER:
+	case ICL_TYPE_INT64:
 	    printfTypes[pftix++] = 0;
 	    printfParms[pfpix++] = alp[pix];
 	    printfTypes[pftix++] = 0;
@@ -472,6 +473,7 @@ static DisplayRecord(outFilep, alp, rsize)
 #endif /* AFS_SGI61_ENV */
 		    break;
 		case ICL_TYPE_HYPER:
+		case ICL_TYPE_INT64:
 		    fprintf(outFilep, "p%d:%x.%x ", i, alp[pix], alp[pix+1]);
 		    break;
 		case ICL_TYPE_FID:
@@ -1646,7 +1648,7 @@ icl_DumpKernel(outFilep, setname)
 	    if (dummy > bufferSize)	/* find biggest log */
 		bufferSize = dummy;
 	    lip = (struct logInfo *) malloc(sizeof(struct logInfo));
-	    bzero((char *)lip, sizeof(*lip));
+	    memset((char *)lip, 0, sizeof(*lip));
 	    lip->nextp = allInfo;
 	    allInfo = lip;
 	    lip->name = (char *) malloc(strlen(tname)+1);
@@ -1664,7 +1666,7 @@ icl_DumpKernel(outFilep, setname)
 	    if (dummy > bufferSize)	/* find biggest log */
 		bufferSize = dummy;
 	    lip = (struct logInfo *) malloc(sizeof(struct logInfo));
-	    bzero((char *)lip, sizeof(*lip));
+	    memset((char *)lip, 0, sizeof(*lip));
 	    lip->nextp = allInfo;
 	    allInfo = lip;
 	    lip->name = (char *) malloc(strlen(tname)+1);
@@ -2184,7 +2186,7 @@ icl_CreateSetWithFlags(name, baseLogp, fatalLogp, flags, outSetpp)
 	states |= ICL_SETF_PERSISTENT;
 
     setp = (struct afs_icl_set *) osi_Alloc(sizeof(struct afs_icl_set));
-    bzero((caddr_t)setp, sizeof(*setp));
+    memset((caddr_t)setp, 0, sizeof(*setp));
     setp->refCount = 1;
     if (states & ICL_SETF_FREED)
 	states &= ~ICL_SETF_ACTIVE;	/* if freed, can't be active */
@@ -2620,7 +2622,7 @@ afs_icl_bulkSetinfo_t *GetBulkSetInfo()
 	    exit (1);
 	}
     }
-    bzero((char *)setInfo, infoSize);
+    memset((char *)setInfo, 0, infoSize);
 
     return setInfo;
 }
@@ -2640,7 +2642,7 @@ afs_icl_bulkLoginfo_t *GetBulkLogInfo()
 	}
     }
 
-    bzero((char *)logInfo, infoSize);
+    memset((char *)logInfo, 0, infoSize);
     return logInfo;
 }
 
